@@ -13,44 +13,88 @@
 ## 2018-11-23 tidy up
 ## 2018-12-21 trap covariates
 ## 2020-10-19 agecov for recoding age effects
+## 2021-04-18 stratified
 ################################################################################
 
-openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL, 
-    sessioncov = NULL, agecov = NULL, dframe = NULL, 
-    contrasts = NULL, initialage = 0, minimumage = 0, maximumage = 1, 
-    CJSp1 = FALSE, ...) {
+openCR.design <- function (capthist, models, type, naive = FALSE, 
+    stratumcov = NULL, sessioncov = NULL, timecov = NULL, agecov = NULL, 
+    dframe = NULL, contrasts = NULL, initialage = 0, minimumage = 0, 
+    maximumage = 1, CJSp1 = FALSE, ...) {
     
 ## Generate design matrix, reduced parameter array, and parameter index array (PIA)
 ## for each parameter
 ## 'capthist' must be of class 'capthist' or 'list'
 
-    findvars <- function (cov, vars, dimcov, scov = FALSE) {
+    findvars.MS <- function (cov, vars, dimcov, scov = FALSE) {
         ## function to add covariates to a design data frame 'dframe'
-        ## uses pad1 and insertdim from utility.R
+        ## based closely on function of same name in 'secr'
+        ## cov may be a dataframe or list of dataframes, one per stratum (R > 1),
+        ## if list, then require predictors to appear in all strata
+        ## uses pad1 from utility.R and insertdim from secr 
+        
         if (is.null(cov) | (length(cov)==0) | (length(vars)==0)) return()
         else {
-            found <- names(cov) %in% vars
-            if (is.data.frame(cov) & any(found)) {
-                found <- names(cov)[found]
-                values <- as.data.frame(cov[,found])
-                values <- secr:::stringsAsFactors(values)  ## updated 2020-12-04
-                names(values) <- found
-                if (length(values)>0) {
-                    for (variable in found) {
-                        vals <- values[,variable]
-                        if (scov) vals <- rep(vals, secondarysessions)
-                        dframe[,variable] <<- secr::insertdim (vals, dimcov, dims)
+            found <- ''
+            if (!is.data.frame(cov)) {
+                ## therefore assume cov is a list, one per stratum
+                ## care required as strata may differ in n, S, K
+                ## solution is to pad to max [n,S,K] over strata
+                if (!is.list(cov) | (R==1))
+                    stop ("irregular covariates; check stratum structure")
+                cov <- lapply(cov, stringsAsFactors)
+                covnames <- lapply(cov, names)
+                varincov <- sapply(covnames, function(nam) vars %in% nam)
+                if (length(vars)>1) found <- vars[apply(varincov,1,all)]
+                else found <- vars[all(varincov)]
+                for (variable in found) {
+                    ## if factor, check all levels the same 
+                    if (any(sapply(cov, function(x) is.factor(x[,variable])))) {
+                        baselevels <- levels(cov[[1]][,variable])
+                        lev <- lapply(cov, function(x) levels(x[,variable]))
+                        if (!all(sapply(lev[-1], function(x) identical(x,baselevels)))) {
+                            print(lev)
+                            stop("covariate factor levels differ between strata")
+                        }
+                    }
+                    ## pad on first dimension... Is this sufficient for e.g. b?
+                    onestratum <- function(x,i) {
+                        xv <- x[,variable]
+                        if (scov) xv <- rep(xv, secondarysessions[[i]])
+                        pad1(xv, dims[dimcov[1]])
+                    }
+                    vals <- mapply(onestratum, cov, 1:length(cov))
+                    vals <- unlist(vals)
+                    dframe[,variable] <<- insertdim (vals, dimcov, dims)
+                }
+            }
+            else
+            {
+                cov <- stringsAsFactors(cov)
+                found <- names(cov) %in% vars
+                if (is.data.frame(cov) & any(found)) {
+                    found <- names(cov)[found]
+                    values <- as.data.frame(cov[,found])
+                    names(values) <- found
+                    if (length(values)>0) {
+                        for (variable in found) {
+                            vals <- values[,variable]
+                            if (scov) vals <- rep(vals, secondarysessions)
+                            dframe[,variable] <<- insertdim (vals, dimcov, dims)
+                        }
                     }
                 }
-                vars <<- vars[!(vars %in% found)]
             }
+            vars <<- vars[!(vars %in% found)]
         }
     }
 
+    # not tested for openCR 2.0.0
     findvars.covtime <- function (covindices, vars) {
-        ## function to add time-specific covariates to a design data frame 'dframe'
+        ## function to add time-specific individual covariates to a 
+        ## design data frame 'dframe'
         ## covindices should be a list
-        dimcov <- c(1,2)   ## animal, secondarysession
+        if (R>1) stop ("time-varying covariates not implemented for stratified designs")
+        dimcov <- c(2,3)   ## animal, secondarysession
         ## covindices is list of numeric or character index vectors, one component per session
         if (length(covindices[[1]]) != J)
             stop ("require one index per primary session")
@@ -84,24 +128,47 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
     npar     <- length(models)                # real parameters
     parnames <- names(models)                 # c('p','phi') for CJS
     vars     <- unique (unlist(sapply (models, all.vars)))
+    MS       <- ms(capthist)    # multistratum
     trps     <- traps(capthist)
     trapcov  <- covariates(trps)
-    used     <- usage(trps)>0
-    nmix     <- secr:::get.nmix(models, capthist, NULL)
-    zcov     <- covariates(capthist)          # individual covariates
-    n        <- nrow(capthist)
-    S        <- ncol(capthist)
-    K        <- if (grepl('secr', type)) nrow(trps) else 1
-    intervals         <- attr(capthist, 'intervals')
-    if (is.null(intervals)) intervals <- rep(1, ncol(capthist)-1)
-    J                 <- sum(intervals>0) + 1 
-    primarysession    <- primarysessions(intervals)
-    secondarysessions <- tabulate(primarysession)
-    firstofsession    <- match(1:J, primarysession)
-    validlevels       <- getvalidlevels (type, parnames, J, CJSp1)
-    
-    # primary.secondary <- 
+    if (MS) {
+        R <- length(capthist)                                    # nstrata
+        n <- max(sapply(capthist, nrow))                         # max over strata
+        S <- max(sapply(capthist, ncol))                         # max over strata
+        K <- if (!grepl('secr', type)) 1
+        else max(sapply(trps, nrow))                             # max over strata
+        stratumlevels <- session(capthist)
+    }
+    else {
+        R <- 1
+        n <- nrow(capthist)
+        S <- ncol(capthist)
+        K <- if (grepl('secr', type)) nrow(trps) else 1
+        stratumlevels <- '1'
+    }
     
+    nmix  <- get.nmix(models)
+    zcov  <- covariates(capthist)            # stratum-specific individual covariates
+
+    if (MS) {
+        intervals <- lapply(capthist, intervals)
+        if (is.null(intervals)) intervals <- lapply(sapply(capthist, ncol)-1, rep, x=1)
+        J                 <- lapply(intervals, function(x) sum(x>0)+1)
+        primarysession    <- lapply(intervals, primarysessions)
+        secondarysessions <- lapply(primarysession, tabulate)
+        getj              <- lapply(J, seq, from = 1)
+        firstofsession    <- mapply(match, getj, primarysession, SIMPLIFY = FALSE )
+        validlevels       <- lapply(J, getvalidlevels, type=type, parnames=parnames, CJSp1=CJSp1)
+    }
+    else {
+        intervals         <- attr(capthist, 'intervals')
+        if (is.null(intervals)) intervals <- rep(1, ncol(capthist)-1)
+        J                 <- sum(intervals>0) + 1 
+        primarysession    <- primarysessions(intervals)
+        secondarysessions <- tabulate(primarysession)
+        firstofsession    <- match(1:J, primarysession)
+        validlevels       <- getvalidlevels (type, parnames, J, CJSp1)
+    }
     #--------------------------------------------------------------------------
 
     if (!grepl('secr', type) & any(.openCRstuff$traplearnedresponses %in% vars))
@@ -115,85 +182,80 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
         warning ("learned response models were re-defined in version 1.3 - check vignette")
     
     #--------------------------------------------------------------------------
-    # session covariates   (primary sessions)
-    if (!is.null(sessioncov)) {
-        scov <- sessioncov  # trick to name a vector 'scov'
-        sessioncov <- as.data.frame(scov)
-        sessioncov <- secr:::stringsAsFactors(sessioncov)  ## updated 2020-12-04
-        if (nrow(sessioncov) != J)
-            stop("number of rows in 'sessioncov' should equal ",
-                 "number of primary sessions")
-    }
-    #--------------------------------------------------------------------------
-    # time covariates   (secondary sessions)
-    if (!is.null(timecov)) {
-        tcov <- timecov  # trick to name a vector 'tcov'
-        timecov <- as.data.frame(tcov)
-        timecov <- secr:::stringsAsFactors(timecov)  ## updated 2020-12-04
-        if (nrow(timecov) != S)
-            stop("number of rows in 'timecov' should equal ",
-                 "number of occasions (secondary sessions")
-    }
-    #--------------------------------------------------------------------------
-    # age covariates   (primary sessions)
-    if (!is.null(agecov)) {
-        acov <- agecov  # trick to name a vector 'acov'
-        agecov <- as.data.frame(acov)
-        agecov <- secr:::stringsAsFactors(agecov)
-        if (nrow(agecov) != (maximumage - minimumage + 1))
-            stop("number of rows in 'agecov' should equal ",
-                "number of possible ages")
-    }
+
+    sessioncov <- stdcovlist(sessioncov, 'scov', R, unlist(J))
+    timecov    <- stdcovlist(timecov, 'tcov', R, if(MS) sapply(capthist, ncol) else ncol(capthist))
+    agecov     <- stdcovlist(agecov, 'acov', R, (maximumage - minimumage + 1))
+    stratumcov <- stdcovlist(stratumcov, 'stratumcov', 1, NULL)
     
     #--------------------------------------------------------------------------
-    dims <- c(n,S,K,nmix)       # virtual dimensions
+    dims <- c(R,n,S,K,nmix)       # virtual dimensions
     dframenrow <- prod(dims)    # number of rows
-    autovars <- c(.openCRstuff$learnedresponses, 'session', 't', 'tt', 'Session', 
-                  'h2', 'h3', 'age', 'Age', 'Age2')
+    autovars <- c(.openCRstuff$learnedresponses, 'stratum', 'session', 't', 
+        'tt', 'Session', 'h2', 'h3', 'age', 'Age', 'Age2')
     #--------------------------------------------------------------------------
     # user-specified dframe
     if (is.null(dframe)) {
-        dframe <- data.frame(dummy=rep(1, dframenrow))
+        dframe <- data.frame(matrix(nrow=dframenrow, ncol=0))
         dframevars <- ""
     }
     else {
         if (nrow(dframe) !=  dframenrow )
-            stop ("dframe should have ", n*S*K*nmix, " rows ( n*S*K*nmix )")
+            stop ("dframe should have ", R*n*S*K*nmix, " rows ( R*n*S*K*nmix )")
         dframevars <- names(dframe)
     }
     #--------------------------------------------------------------------------
+    # stratum
+    dframe$stratum <- factor( insertdim (stratumlevels, 1, dims),
+        levels = stratumlevels)
+    #--------------------------------------------------------------------------
+    if (MS) {
+        sess <- mapply(rep, x = getj, times = secondarysessions, SIMPLIFY = FALSE)
+        sess <- unlist(lapply(sess, pad1, S))
+        dframe$session <- factor(secr::insertdim (sess,c(3,1),dims))
+        dframe$Session <- secr::insertdim (sess-1,c(3,1),dims)
+        dframe$tt <- factor(secr::insertdim (1:S, c(3,1), dims))
+        if ('t' %in% vars) {
+            dframe$t <- dframe$session
+        }
+    }
+    else {
+        dframe$session <- factor(secr::insertdim (rep(1:J, secondarysessions), 3, dims))
+        dframe$Session <- secr::insertdim (rep(0:(J-1), secondarysessions), 3, dims)
+        dframe$tt <- factor(secr::insertdim (1:S, 3, dims))
+        ## t as synonym of session
+        if ('t' %in% vars) {
+            dframe$t <- factor(secr::insertdim (rep(1:J, secondarysessions), 3, dims))
+        }
+    }
 
-    dframe$session <- factor(secr::insertdim (rep(1:J, secondarysessions), 2, dims))
-    dframe$Session <- secr::insertdim (rep(0:(J-1), secondarysessions), 2, dims)
-    dframe$tt <- factor(secr::insertdim (1:S, 2, dims))
-    
-    ## t as synonym of session
-    if ('t' %in% vars) {
-        dframe$t <- factor(secr::insertdim (rep(1:J, secondarysessions), 2, dims))
-    }
-    ## firstage <- as.numeric(grepl('CJS', type))
-    # firstage <- 0 # replaced by minimumage
-    
-    # rearranged 2020-10-19
     if (any(c('age','Age','Age2', names(agecov)) %in% vars)) {
-        age <- age.matrix(capthist, initialage, minimumage, maximumage)
-    }
-    if ('age' %in% vars) {
-        dframe$age <- factor(secr::insertdim (factor(age), 1:2, dims))
-    } 
-    if ('Age' %in% vars) {
-        dframe$Age <- secr::insertdim (age, 1:2, dims)
-    } 
-    if ('Age2' %in% vars) {
-        dframe$Age2 <- secr::insertdim (age^2, 1:2, dims)
-    } 
-    for (i in names(agecov)) {
-        if (i %in% vars) {
-            agecovi <- agecov[,i][age-minimumage+1]
-            dframe[,i] <- secr::insertdim (agecovi, 1:2, dims)
+        
+        if (MS) {
+            temp <- lapply(capthist, age.matrix, initialage, minimumage, maximumage)
+            temp <- lapply(temp, padarray, c(n,S))
+            age <- unlist(temp)
+        }
+        else {
+            age <- age.matrix(capthist, initialage, minimumage, maximumage) # one stratum
+        }
+        
+        if ('age' %in% vars) {
+            dframe$age <- factor(secr::insertdim (factor(age), c(2,3,1), dims))
+        } 
+        if ('Age' %in% vars) {
+            dframe$Age <- secr::insertdim (age, c(2,3,1), dims)
+        } 
+        if ('Age2' %in% vars) {
+            dframe$Age2 <- secr::insertdim (age^2, c(2,3,1), dims)
+        } 
+        for (i in names(agecov)) {
+            if (i %in% vars) {
+                agecovi <- agecov[,i][age-minimumage+1]
+                dframe[,i] <- secr::insertdim (agecovi, c(2,3,1), dims)
+            }
         }
     }
-    
     #--------------------------------------------------------------------------
 
     ## behavioural response fields
@@ -216,13 +278,17 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
         if (naive) dframe$bsession <- rep(FALSE, dframenrow)
         else {
             prevcapt <- function(x) {
-                #prevcapt1 <- function(x) c(FALSE, cumsum(x[-S])>0)  BUG 2018-11-22
                 prevcapt1 <- function(x) c(FALSE, cumsum(x[-length(x)])>0)
                 ## apply within each primary session
                 unlist(lapply( split(x, primarysession), prevcapt1))
             }
-            temp <- makeb(capthist)
-            dframe$bsession <- secr::insertdim (as.vector(temp), c(1,2), dims)
+            if (MS) {
+                temp <- lapply(capthist, makeb)
+                temp <- lapply(temp, padarray, c(n,S))
+                temp <- unlist(temp)
+            }
+            else temp <- makeb(capthist)  # one stratum
+            dframe$bsession <- secr::insertdim (as.vector(temp), c(2,3,1), dims)
         }
     }
     
@@ -235,8 +301,13 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
                 ## apply within each primary session
                 unlist(lapply( split(x, primarysession), prevcapt1))
             }
-            temp <- makeb(capthist)
-            dframe$Bsession <- secr::insertdim (as.vector(unlist(temp)), c(1,2), dims)
+            if (MS) {
+                temp <- lapply(capthist, makeb)
+                temp <- lapply(temp, padarray, c(n,S))
+                temp <- unlist(temp)
+            }
+            else temp <- makeb(capthist)  # one stratum
+            dframe$Bsession <- secr::insertdim (as.vector(unlist(temp)), c(2,3,1), dims)
         }
     }
     
@@ -245,8 +316,13 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
         if (naive) dframe$b <- rep(FALSE, dframenrow)
         else {
             prevcapt <- function(x) c(FALSE, cumsum(x[-length(x)])>0)
-            temp <- makeb(capthist)
-            dframe$b <- secr::insertdim (as.vector(temp), c(1,2), dims)
+            if (MS) {
+                temp <- lapply(capthist, makeb)
+                temp <- lapply(temp, padarray, c(n,S))
+                temp <- unlist(temp)
+            }
+            else temp <- makeb(capthist)  # one stratum
+            dframe$b <- secr::insertdim (as.vector(temp), c(2,3,1), dims)
         }
     }
     
@@ -262,27 +338,18 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
                 ## EITHER same primary as ct and later OR next primary
                 (capt1 & (primarysession %in% pcapt)) | (primarysession %in% (pcapt+1))
             }
-            temp <- makeb(capthist)
-            dframe$B <- secr::insertdim (as.vector(temp), c(1,2), dims)
+            if (MS) {
+                temp <- lapply(capthist, makeb)
+                temp <- lapply(temp, padarray, c(n,S))
+            }
+            else temp <- makeb(capthist)  # one stratum
+            dframe$B <- secr::insertdim (as.vector(temp), c(2,3,1), dims)
         }
     }
-    # old Bsession
-    # if ('B' %in% vars) {
-    #     if (naive) dframe$B <- rep(FALSE, dframenrow)
-    #     else {
-    #         prevcapt <- function(x) c(FALSE, x[-J]>0)
-    #         temp <- makebJ(capthist)
-    #         dframe$B <- secr::insertdim (as.vector(temp), c(1,2), dims)
-    #     }
-    # }
-    #------------------------------------------------
-    
-        
     #------------------------------------------------
     ## individual trap-specific responses
     
     makebk <- function (caphist) {     
-        # condition added 2016-10-01
         if (nrow(caphist)==0) 
             array(dim = c(0,S,K))
         else {
@@ -300,8 +367,12 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
                 ## apply within each primary session
                 unlist(lapply( split(x, primarysession), prevcapt1))
             }
-            temp <- makebk(capthist) 
-            dframe$bksession <- secr::insertdim(temp, 1:3, dims)  
+            if (MS) {
+                temp <- lapply(capthist, makebk)
+                temp <- lapply(temp, padarray, c(n,S,K))
+            }
+            else temp <- makebk(capthist)  # one stratum
+            dframe$bksession <- secr::insertdim(temp, c(2,3,4,1), dims)  
         }
     }
     
@@ -310,8 +381,12 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
         if (naive) dframe$Bksession <- rep(FALSE, dframenrow)
         else {
             prevcapt <- function(x) c(FALSE, x[-S]>0)
-            temp <- makebk(capthist)  # one session
-            dframe$Bksession <- secr::insertdim(temp, 1:3, dims)
+            if (MS) {
+                temp <- lapply(capthist, makebk)
+                temp <- lapply(temp, padarray, c(n,S,K))
+            }
+            else temp <- makebk(capthist)  # one stratum
+            dframe$Bksession <- secr::insertdim(temp, c(2,3,4,1), dims)
         }
     }
     
@@ -320,8 +395,12 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
         if (naive) dframe$bk <- rep(FALSE, dframenrow)
         else {
             prevcapt <- function(x) c(FALSE, cumsum(x[-length(x)])>0)
-            temp <- makebk(capthist) 
-            dframe$bk <- secr::insertdim(temp, 1:3, dims)  
+            if (MS) {
+                temp <- lapply(capthist, makebk)
+                temp <- lapply(temp, padarray, c(n,S,K))
+            }
+            else temp <- makebk(capthist)  # one stratum
+            dframe$bk <- secr::insertdim(temp, c(2,3,4,1), dims)  
         }
     }
     
@@ -342,8 +421,12 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
         if (naive) dframe$k <- rep(FALSE, dframenrow)
         else {
             prevcapt <- function(x) c(FALSE, cumsum(x[-length(x)])>0)
-            temp <- makek(capthist)
-            dframe$k <- secr::insertdim(temp, 2:3, dims)
+            if (MS) {
+                temp <- lapply(capthist, makek)
+                temp <- lapply(temp, padarray, c(S,K))
+            }
+            else temp <- makek(capthist)  # one stratum
+            dframe$k <- secr::insertdim(temp, c(3,4,1), dims)
         }
     }
 
@@ -356,8 +439,12 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
                 ## apply within each primary session
                 unlist(lapply( split(x, primarysession), prevcapt1))
             }
-            temp <- makek(capthist)
-            dframe$ksession <- secr::insertdim(temp, 2:3, dims)
+            if (MS) {
+                temp <- lapply(capthist, makek)
+                temp <- lapply(temp, padarray, c(S,K))
+            }
+            else temp <- makek(capthist)  # one stratum
+            dframe$ksession <- secr::insertdim(temp, c(3,4,1), dims)
         }
     }
 
@@ -370,16 +457,19 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
                 ## apply within each primary session
                 unlist(lapply( split(x, primarysession), prevcapt1))
             }
-            temp <- makek(capthist)
-            dframe$Ksession <- secr::insertdim(temp, 2:3, dims)
+            if (MS) {
+                temp <- lapply(capthist, makek)
+                temp <- lapply(temp, padarray, c(S,K))
+            }
+            else temp <- makek(capthist)  # one stratum
+            dframe$Ksession <- secr::insertdim(temp, c(3,4,1), dims)
         }
     }
-
     #------------------------------------------------
     ## h2 or h3
     if (nmix > 1) {
         mixture <- paste('h',nmix,sep='')
-        dframe[,mixture] <- secr::insertdim(factor(1:nmix), 4, dims)
+        dframe[,mixture] <- secr::insertdim(factor(1:nmix), 5, dims)
     }
 
     #--------------------------------------------------------------------------
@@ -387,13 +477,16 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
     vars <- vars[!(vars %in% c(autovars, names(agecov), dframevars))]
 
     #--------------------------------------------------------------------------
-    # add zcov, sessioncov etc.
 
-    findvars (zcov, vars, 1)
-    findvars (timecov, vars, 2)
-    findvars (trapcov, vars, 3)  ## 2018-12-21
-    findvars (sessioncov, vars, 2, scov = TRUE)  ## expands correctly 2018-05-07
-    findvars (agecov, vars, 1, scov = TRUE)      ## new 2020-10-19
+    # add miscellaneous covariates
+
+    findvars.MS (stratumcov, vars, 1)
+    findvars.MS (zcov, vars, 2)
+    findvars.MS (timecov, vars, 3)
+    findvars.MS (trapcov, vars, 4) 
+    findvars.MS (sessioncov, vars, 3, scov = TRUE)  ## expands correctly 2018-05-07
+    findvars.MS (agecov, vars, 2, scov = TRUE)      ## new 2020-10-19
+    
     tvc <- timevaryingcov(capthist)
     if (!is.null(tvc) & (length(vars)>0)) {
         findvars.covtime (tvc, vars)
@@ -408,7 +501,6 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
         }
         stop ("covariate(s) ", paste(vars,collapse=","), " not found")
     }
-
     make.designmatrix <- function (formula, prefix, ...) {
      # combine formula and dframe to generate design matrix
         if (is.null(formula)) {
@@ -451,7 +543,7 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
     
     #--------------------------------------------------------------------
     # PIA = Parameter Index Array
-    #       index to row of parameterTable for a given n,s,nmix
+    #       index to row of parameterTable for a given R,n,s,K,nmix
     # dim(parameterTable) = c(uniqueparcomb, npar)
     #       index to row of designMatrix for each real parameter
     #--------------------------------------------------------------------
@@ -467,23 +559,33 @@ openCR.design <- function (capthist, models, type, naive = FALSE, timecov = NULL
     #--------------------------------------------------------------------
 
     if (grepl('secr', type)) {
-        if ((!is.null(unlist(used))) & (length(used)>0)) {
-            allused <- unlist(used)
-            if (!is.null(allused)) {
-                if (any(!allused)) {
-                    PIA[ , , ,] <- PIA[ , , ,] * rep(rep(t(used),rep(n,S*K)),nmix)
+        for (stratum in 1:R) {
+            if (MS) {
+                used <- usage(trps)[[stratum]]
+            }
+            else {
+                used <- usage(trps)
+            }
+            if ((!is.null(unlist(used))) & (length(used)>0)) {
+                allused <- unlist(used)
+                if (!is.null(allused)) {
+                    if (any(!allused)) {
+                        used <- padarray(t(used), c(S,K))
+                        PIA[stratum,,,,] <- PIA[stratum,,,,] * rep(rep(used,rep(n,S*K)),nmix)
+                    }
                 }
             }
         }
     }
     individual <- individualcovariates(PIA)
     #--------------------------------------------------------------------
+
     list(designMatrices = designMatrices, parameterTable = parameterTable, PIA = PIA,
          PIAJ = PIAJ, validlevels = validlevels, individual = individual)
 }
 ############################################################################################
 
-## PIA has dim = c(nc, ss, kk, xx)
+## PIA has dim = c(R, nc, ss, kk, xx)
 ## primarysession gives primary session (j) for each secondary session (s)
 # returns list with components 
 #    lookup - matrix of unique rows, each row ss * kk long, containing indices as in PIA to 
@@ -520,24 +622,32 @@ njxlookup <- function (PIA, primarysession) {
 # direct index to session PIA in njx array 2018-02-10
 # slow - must be a better way 2018-04-11
 njx <- function (PIA, primarysession) {
-    n <- dim(PIA)[1]
-    J <- max(primarysession)
-    xx <- dim(PIA)[4]
-    # why all this complexity? 2018-04-11
-    # dims <- lapply(dim(PIA), seq, from=1)
-    # names(dims) <- c('n','s','k','x')
-    # df <- data.frame(pia = as.numeric(PIA), do.call(expand.grid, dims))
-    # names(df) <- c('pia', 'n','s','k','x')
-    # df$j <- formatC(primarysession[df$s], width=3, flag="0")
-    # df$n <- formatC(df$n, width=4, flag="0")
-    # splitter <- apply(df[,c('x', 'j', 'n')], 1, paste, collapse='.')  # deliberately reverse order
-    # splitdf <- split(df[,c('pia','s')], splitter)
-    # piaval <- sapply(splitdf, function(x) x$pia[which.min(!(x$pia != 0))])
-    # array(piaval, dim = c(n, J, xx))
-
-    # instead 2018-04-11   
-    s1 <- match(1:J, primarysession)
-    piaj <- PIA[,s1,1,]
-    array(piaj, dim = c(n,J,xx))
+    R <- dim(PIA)[1]
+    n <- dim(PIA)[2]
+    S <- dim(PIA)[3]
+    xx <- dim(PIA)[5]
+    maxJ <- max(unlist(primarysession))
+    out <- array(1, dim = c(R,n,maxJ,xx))
+    if (R>1) {
+        for (i in 1:R) {
+            J <- max(primarysession[[i]])
+            s1 <- pad1(match(1:J, primarysession[[i]]), maxJ)
+            out[i,,,] <- PIA[i,,s1,1,]
+        }
+    }
+    else {
+        J <- max(primarysession)
+        s1 <- match(1:J, primarysession)
+        out[1,,,] <- PIA[1,,s1,1,]
+    }
+    out
 }
-
+# 
+# n <- dim(PIA)[1]
+# J <- max(primarysession)
+# xx <- dim(PIA)[4]
+# # instead 2018-04-11   
+# s1 <- match(1:J, primarysession)
+# piaj <- PIA[,s1,1,]
+# array(piaj, dim = c(n,J,xx))
+# }
\ No newline at end of file
