@@ -3,7 +3,6 @@
 
 using namespace Rcpp;
 using namespace RcppParallel;
-
 #define fuzz 1e-200
 #define huge 1e10
 
@@ -58,10 +57,10 @@ double gbinom(int count, int size, double p, int uselog)
 //--------------------------------------------------------------------------
 
 double gbinomFP (int count, double size, double p, int uselog)
-// allow non integer size for Binomial
+    // allow non integer size for Binomial
 {
     return ( lgamma(size+1) - lgamma(size-count+1) - lgamma(count+1) +
-             count * log(p) + (size - count) * log (1-p) );
+        count * log(p) + (size - count) * log (1-p) );
 }
 //--------------------------------------------------------------------------
 
@@ -70,9 +69,9 @@ double gbinomFP (int count, double size, double p, int uselog)
 // a probability otherwise
 
 double pski ( int binomN,
-              int count,
-              double Tski,
-              double g) {
+    int count,
+    double Tski,
+    double g) {
     
     double result = 1.0;
     
@@ -101,7 +100,7 @@ double pski ( int binomN,
         result = gbinom (count, binomN, g, 0);
     }
     else stop("binomN < -1 not allowed");  // code multi -2 separately
-
+    
     return (result);
 }
 //--------------------------------------------------------------------------
@@ -110,32 +109,52 @@ double pski ( int binomN,
 double dkm (int k, int m, RMatrix<double> traps, RMatrix<double> mask)
 {
     return(sqrt((traps(k,0) - mask(m,0)) * (traps(k,0) - mask(m,0)) +
-		(traps(k,1) - mask(m,1)) * (traps(k,1) - mask(m,1))));
+        (traps(k,1) - mask(m,1)) * (traps(k,1) - mask(m,1))));
+}
+//--------------------------------------------------------------------------
+
+// rectangular distance between two points given by row k in traps and row m in mask
+double dkmrect (int k, int m, RMatrix<double> traps, RMatrix<double> mask)
+{
+    return(std::max(fabs(traps(k,0) - mask(m,0)), fabs(traps(k,1) - mask(m,1))));
 }
 //--------------------------------------------------------------------------
 
 // parameters in openval ordered g0, phi, f, N, sigma, pmix 
 
-// hazard detection functions 14-19
+// hazard detection functions 14-20
 double hfn
     (int k, int m, int c, 
-     const RMatrix<double> openval,  
-     const RMatrix<double> traps,
-     const RMatrix<double> mask, 
-     int sigmai, 
-     int detectfn)
+        const RMatrix<double> openval,  
+        const RMatrix<double> traps,
+        const RMatrix<double> mask, 
+        int sigmai, 
+        int detectfn)
 {
     double d;
     double sigma;
     double z = 1;
     sigma =  openval(c, sigmai);
-    d = dkm(k, m, traps, mask);
+    if (detectfn == 20) {
+        d = dkmrect(k, m, traps, mask);
+    }
+    else {
+        d = dkm(k, m, traps, mask);
+    }
+    
     if (detectfn == 14) { 
         return (openval(c,0) * exp(-d*d/2/sigma/sigma));                   // HHN
     }
     else if (detectfn == 16) {
         return (openval(c,0) * exp(-d/sigma));                             // HEX
     }
+    else if (detectfn == 20) {
+        if (d<=sigma)
+            return (openval(c,0));
+        else
+            return (0);                                                    // HPX
+            // return (1e-5);                                              // arbitrary to avoid NA
+    }
     else {
         z =  openval(c,sigmai+1);
         if (detectfn == 15) {
@@ -155,160 +174,47 @@ double hfn
 }
 //--------------------------------------------------------------------------
 
-void convolvemq (
-        int    mm,                    // number of points on mask 
-        int    kn,                    // number of points on kernel
-        int    j,                     // session number 1..jj 
-        int    edgecode,              // 0 none, no action; 1 wrapped, no action; 2 normalize truncated kernel
-        const  RMatrix<int> &mqarray, // input [& 2020-10-31]
-        std::vector<double> &kernelp, // p(move|dx,dy) for points in kernel 
-        std::vector<double> &pjm      // return value
-)
+// hazard detection functions 14-20
+double hfnd
+    (int k, int m, int c, 
+        const RMatrix<double> openval,  
+        const RMatrix<double> distmat,
+        int sigmai, 
+        int detectfn)
 {
-    int m, q, mq;
-    double sump;
-    std::vector<double> workpjm(mm);
-    
-    // convolve movement kernel and pjm... 
-    for (m = 0; m < mm; m++) {
-        if (edgecode == 2) {
-            // 2020-10-29 adjust for edge-truncated kernel cf convolvemqold
-            sump = 0;
-            for (q=0; q < kn; q++) {           // over movement kernel 
-                if (mqarray(m,q) >= 0) {       // post-dispersal site is within mask 
-                    sump += kernelp[kn * (j-1) + q];
-                }
-            }
-        }
-        else {
-            sump = 1.0;
-        }
-        if (sump>0) {
-            // over movement kernel 
-            for (q=0; q < kn; q++) {           
-                mq = mqarray(m,q);  
-                // post-dispersal site is within mask 
-                if (mq >= 0) {                 
-                    // probability of this move 
-                    workpjm[mq] += pjm[m] * kernelp[kn * (j-1) + q] / sump;   
-                }
-            }
-        }
+    double d;
+    double sigma;
+    double z = 1;
+    sigma =  openval(c, sigmai);
+    d = distmat(k,m);
+    if (detectfn == 14) { 
+        return (openval(c,0) * exp(-d*d/2/sigma/sigma));                   // HHN
     }
-    for (m = 0; m < mm; m++) {
-        pjm[m] = workpjm[m];
+    else if (detectfn == 16) {
+        return (openval(c,0) * exp(-d/sigma));                             // HEX
     }
-}
-//--------------------------------------------------------------------------
-
-void fillkernelp (int kn, 
-                  int jj, 
-                  int kerneltype, 
-                  double cellsize,
-                  const RMatrix<int> kernel, 
-                  const RVector<int> moveargsi, 
-                  //const CharacterVector fnname,
-                  const String fnname,
-                  const std::vector<double> &moveargs, 
-                  std::vector<double> &kernelp) {
-    int j,k;
-    double r,r2,a,a2,b;
-    NumericVector p;
-    std::vector<double> sumj(jj);
-    for (j = 0; j < (jj-1); j++) sumj[j] = 0;
-    for (k = 0; k < kn; k++) {
-        r2 = (kernel[k]*kernel[k] + kernel[k+kn]*kernel[k+kn]) * cellsize * cellsize;
-        r = sqrt(r2);
-        for (j = 0; j < (jj-1); j++) {
-            if (kerneltype == 0) {         // Gaussian kernel 
-                a2 = moveargs[j] * moveargs[j];
-                kernelp[j * kn + k] = exp(-r2 / 2 / a2);
-	    }
-            else if (kerneltype == 1) {   // Negative exponential kernel 
-                a = moveargs[j];
-                kernelp[j * kn + k] = exp(-r / a);
-	    }
-            else if (kerneltype == 3) {   // 2-D t kernel 
-                a2 = moveargs[j] * moveargs[j];
-                b = moveargs[j+jj] + 1;
-                kernelp[j * kn + k] = (b-1) / M_PI / a2 / pow(1 + r*r/a2, b);
-            }
-            else if (kerneltype == 2) {   // User kernel 
-                // call R function from C++
-                Environment env = Environment::global_env();
-                Function f = env[fnname];
-                if (moveargsi[1]>0)
-                    p = f(r, moveargs[j], moveargs[j+jj]);
-                else if (moveargs[0]>0)
-                    p = f(r, moveargs[j]);
-                else 
-                    p = f(r);
-                kernelp[j * kn + k] = p[0];
-                // Rprintf(" k %4d j %4d  kernelp[j * kn + k] %8.6f\n",  k,j,kernelp[j * kn + k]); 
-            }
-            else if (kerneltype == 4) {  // uniform kernel 
-                kernelp[j * kn + k] = 1.0 / kn;
-            }
-            else stop("unrecognised kerneltype");
-            sumj[j] += kernelp[j * kn + k];
-        }
+    else if (detectfn == 20) {
+        if (d<=sigma)
+            return (openval(c,0));
+        else
+            return (0);                                                    // HPX
+        // return (1e-5);                                              // arbitrary to avoid NA
     }
-    // normalise 
-    for (k = 0; k < kn; k++) {
-        for (j = 0; j < (jj-1); j++) {
-            kernelp[j * kn + k] = kernelp[j * kn + k] / sumj[j];
+    else {
+        z =  openval(c,sigmai+1);
+        if (detectfn == 15) {
+            return (openval(c,0) * (1 - exp(-pow(d/sigma, -z))));          // HHR
         }
-    }
-}
-//--------------------------------------------------------------------------
-
-// version with no option for user function (R calls prohibited in RcppParallel)
-void fillkernelparallel (int kn, 
-                  int jj, 
-                  int kerneltype, 
-                  double cellsize,
-                  const RMatrix<int> kernel, 
-                  const RVector<int> moveargsi, 
-                  const std::vector<double> &moveargs, 
-                  std::vector<double> &kernelp) {
-    int j,k;
-    double r,r2,a,a2,b;
-    std::vector<double> p(jj);
-    std::vector<double> sumj(jj);
-    for (j = 0; j < (jj-1); j++) sumj[j] = 0;
-    for (k = 0; k < kn; k++) {
-        r2 = (kernel[k]*kernel[k] + kernel[k+kn]*kernel[k+kn]) * cellsize * cellsize;
-        r = sqrt(r2);
-        for (j = 0; j < (jj-1); j++) {
-            if (kerneltype == 0) {        // Gaussian kernel 
-                a2 = moveargs[j] * moveargs[j];
-                kernelp[j * kn + k] = exp(-r2 / 2 / a2);
-	    }
-            else if (kerneltype == 1) {   // Negative exponential kernel 
-		a = moveargs[j];
-                kernelp[j * kn + k] = exp(-r / a);
-	    }
-            else if (kerneltype == 2) {   // User kernel 
-                // cannot call R function from RcppParallel worker
-                stop("cannot call R function from RcppParallel worker; try ncores = 1");
-            }
-            else if (kerneltype == 3) {   // 2-D t kernel 
-                a2 = moveargs[j] * moveargs[j];
-                b = moveargs[j+jj] + 1;
-                kernelp[j * kn + k] = (b-1) / M_PI / a2 / pow(1 + r2/a2, b);
-            }
-            else if (kerneltype == 4) {   // uniform kernel 
-                kernelp[j * kn + k] = 1.0 / kn;
-            }
-            else stop("unrecognised kerneltype");
-            sumj[j] += kernelp[j * kn + k];
+        else if (detectfn == 17) {
+            return (openval(c,0) * exp(-(d-z)*(d-z) / 2 / sigma / sigma)); // HAN
         }
-    }
-    // normalise 
-    for (k = 0; k < kn; k++) {
-        for (j = 0; j < (jj-1); j++) {
-            kernelp[j * kn + k] = kernelp[j * kn + k] / sumj[j];
+        else if (detectfn == 18) {
+            return (openval(c,0) * R::pgamma(d,z,sigma/z,0,0));            // HCG
         }
+        else if (detectfn == 19) {
+            return (openval(c,0) * exp(- pow(d /sigma , z)));              // HVP
+        }
+        else stop("detectfn not allowed in openCR");
     }
 }
 //--------------------------------------------------------------------------
@@ -316,9 +222,9 @@ void fillkernelparallel (int kn,
 
 
 void getp (int n, int x, int nc, int ss, 
-           const RMatrix<double> openval,  
-           const RVector<int> PIA, 
-           std::vector<double> &p) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIA, 
+    std::vector<double> &p) {
     // column 1 
     int s;
     for (s = 0; s < ss; s++) {
@@ -328,10 +234,10 @@ void getp (int n, int x, int nc, int ss,
 //--------------------------------------------------------------------------
 
 void getphij (int n, int x, int nc, int jj, 
-              const RMatrix<double> openval,  
-              const RVector<int> PIAJ,
-              const RVector<double> intervals, 
-              std::vector<double> &phij) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    const RVector<double> intervals, 
+    std::vector<double> &phij) {
     // column 2 
     int j;
     double phi;
@@ -346,10 +252,10 @@ void getphij (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getmoveargs (int n, int x, int nc, int jj, 
-		  const RMatrix<double> openval,  
-		  const RVector<int> PIAJ,
-		  const RVector<int> moveargsi,
-		  std::vector<double> &moveargs) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    const RVector<int> moveargsi,
+    std::vector<double> &moveargs) {
     // column moveargsi (and maybe moveargsi + 1) 
     int j;
     for (j = 0; j < (jj-1); j++) {
@@ -364,9 +270,9 @@ void getmoveargs (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getpj (int n, int x, int nc, int jj, 
-            const RMatrix<double> openval,  
-            const RVector<int> PIAJ,
-            std::vector<double> &pj) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &pj) {
     // column 2 
     int j;
     for (j = 0; j < jj; j++) {
@@ -376,9 +282,9 @@ void getpj (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getg (int type, int n, int x, int nc, int jj, 
-           const RMatrix<double> openval,  
-           const RVector<int> PIAJ,
-           std::vector<double> &g) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &g) {
     // column 4 
     int j;
     for (j = 0; j < jj; j++) {
@@ -391,18 +297,18 @@ void getg (int type, int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getfj (int n, int x, int nc, int jj, 
-            const RMatrix<double> openval,  
-            const RVector<int> PIAJ,
-            const RVector<double> intervals, 
-            std::vector<double> &phij,
-            std::vector<double> &fj) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    const RVector<double> intervals, 
+    std::vector<double> &phij,
+    std::vector<double> &fj) {
     // column 3 
     int j;
     double f,phi;
     for (j = 0; j < (jj-1); j++) {
         // jj-1 because one fewer intervals than primary sessions  
         f = openval(PIAJ[i3(n, j, x, nc, jj )]-1, 2); 
-	phi = exp(log(phij[j]) / intervals[j]);
+        phi = exp(log(phij[j]) / intervals[j]);
         // adjust for interval duration  
         fj[j] = exp(log(phi+f) * intervals[j]) - phij[j];  
     }
@@ -411,10 +317,10 @@ void getfj (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getlj (int n, int x, int nc, int jj, 
-            const RMatrix<double> openval,  
-            const RVector<int> PIAJ,
-            const RVector<double> intervals, 
-            std::vector<double> &lj) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    const RVector<double> intervals, 
+    std::vector<double> &lj) {
     // column 3 
     int j;
     double l;
@@ -429,10 +335,10 @@ void getlj (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getgaml (int n, int x, int nc, int jj, 
-              const RMatrix<double> openval,  
-              const RVector<int> PIAJ,
-              const RVector<double> intervals, 
-              std::vector<double> &gam) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    const RVector<double> intervals, 
+    std::vector<double> &gam) {
     // column 3 
     int j;
     double phij;
@@ -449,10 +355,10 @@ void getgaml (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getgamj (int n, int x, int nc, int jj, 
-              const RMatrix<double> openval,  
-              const RVector<int> PIAJ,
-              const RVector<double> intervals, 
-              std::vector<double> &gamj) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    const RVector<double> intervals, 
+    std::vector<double> &gamj) {
     // column 3 
     int j;
     double gam;
@@ -465,9 +371,9 @@ void getgamj (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getkapj (int n, int x, int nc, int jj, 
-              const RMatrix<double> openval,  
-              const RVector<int> PIAJ,
-              std::vector<double> &kapj) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &kapj) {
     // column 3 
     int j;
     for (j = 1; j < jj; j++) {
@@ -483,9 +389,9 @@ void getkapj (int n, int x, int nc, int jj,
 // }
 
 void getbeta0 (int n, int x, int nc, int jj, 
-               const RMatrix<double> openval,  
-               const RVector<int> PIAJ,
-               std::vector<double> &beta) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &beta) {
     // column 3 
     int j;
     double sumbeta = 0;
@@ -502,10 +408,10 @@ void getbeta0 (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void gettau (int n, int x, int nc, int jj,
-             const RMatrix<double> openval,  
-             const RVector<int> PIAJ,
-             std::vector<double> &tau,
-             int M) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &tau,
+    int M) {
     // column 5 
     int j;
     double sumtau = 0;
@@ -524,9 +430,9 @@ void gettau (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getDj (int n, int x, int nc, int jj, 
-            const RMatrix<double> openval,  
-            const RVector<int> PIAJ,
-            std::vector<double> &Dj) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &Dj) {
     // column 3 
     int j;
     for (j = 0; j < jj; j++) {
@@ -537,11 +443,11 @@ void getDj (int n, int x, int nc, int jj,
 
 // per capita recruitment cf Link & Barker 2005, Schwarz 'Gentle Intro'
 void getbetaf (int n, int x, int nc, int jj, 
-               const RMatrix<double> openval,  
-               const RVector<int> PIAJ,
-               std::vector<double> &phij,
-               const RVector<double> intervals, 
-               std::vector<double> &beta) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &phij,
+    const RVector<double> intervals, 
+    std::vector<double> &beta) {
     int j;
     double sumbeta = 1;
     std::vector<double> d(jj);
@@ -564,17 +470,17 @@ void getbetaf (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getbetal (int n, int x, int nc, int jj, 
-               const RMatrix<double> openval,  
-               const RVector<int> PIAJ,
-               std::vector<double> &phij,
-               const RVector<double> intervals, 
-               std::vector<double> &beta) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ,
+    std::vector<double> &phij,
+    const RVector<double> intervals, 
+    std::vector<double> &beta) {
     int j;
     double sumbeta = 1;
     std::vector<double> d(jj);
     std::vector<double> fj(jj);
     std::vector<double> lambdaj(jj);
-
+    
     getlj (n, x, nc, jj, openval, PIAJ, intervals, lambdaj);
     for (j=0; j<jj; j++) {
         if (lambdaj[j] < phij[j])  
@@ -599,18 +505,18 @@ void getbetal (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getbetag (int n, int x, int nc, int jj, 
-               const RMatrix<double> openval,  
-               const RVector<int> PIAJ, 
-               std::vector<double> &phij,
-               const RVector<double> intervals, 
-               std::vector<double> &beta) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ, 
+    std::vector<double> &phij,
+    const RVector<double> intervals, 
+    std::vector<double> &beta) {
     int j;
     double sumbeta = 1;
     std::vector<double> d(jj);
     std::vector<double> fj(jj);
     std::vector<double> gamj(jj);
     getgamj (n, x, nc, jj, openval, PIAJ, intervals, gamj);
-
+    
     for (j=1; j<jj; j++) {
         if (gamj[j] <= 0)  
             fj[j-1] = 0;    
@@ -618,7 +524,7 @@ void getbetag (int n, int x, int nc, int jj,
             fj[j-1] = phij[j-1] * (1/gamj[j] - 1);   // Pradel 1996 p 708 corrected!
     }
     fj[jj-1] = 0;
-
+    
     d[0] = 1;
     for (j = 1; j < jj; j++) {
         d[j] = d[j-1] * (phij[j-1] + fj[j-1]);
@@ -636,37 +542,37 @@ void getbetag (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getbetak (int n, int x, int nc, int jj, 
-               const RMatrix<double> openval,  
-               const RVector<int> PIAJ, 
-               std::vector<double> &phij,
-               std::vector<double> &beta) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ, 
+    std::vector<double> &phij,
+    std::vector<double> &beta) {
     int i,j;
     std::vector<double> tau(jj);
     std::vector<double> pj(jj);
     std::vector<double> fprod(jj);
     std::vector<double> fj(jj);
     std::vector<double> kapj(jj);
-
+    
     getkapj (n, x, nc, jj, openval, PIAJ, kapj);
     getpj (n, x, nc, jj, openval, PIAJ, pj);
-
+    
     tau[0] = 1/pj[0];
     for (j=0; j<(jj-1); j++) {
         fj[j] = (kapj[j+1] - kapj[j]/pj[j] * (1 - pj[j]) * phij[j] * pj[j+1]) / 
-	    (tau[j] * pj[j+1]);
+            (tau[j] * pj[j+1]);
         tau[j+1] = tau[0];  // get next tau
-	for (i=0; i<(j+1); i++) tau[j+1] *= (phij[i] + fj[i]);
+        for (i=0; i<(j+1); i++) tau[j+1] *= (phij[i] + fj[i]);
     }
     for (j=1; j<(jj-1); j++) {
-	fprod[j] = fj[j];
-	for (i=0; i<j; i++) fprod[j] *= (phij[i] + fj[i]);
+        fprod[j] = fj[j];
+        for (i=0; i<j; i++) fprod[j] *= (phij[i] + fj[i]);
     }
     beta[0] = 1 + fj[0];
     for (j=1; j<(jj-1); j++) beta[0] += fprod[j];
     beta[0] = 1/beta[0];
     beta[1] = beta[0] * fj[0];
     for (j=1; j<(jj-1); j++) {
-	beta[j+1] = beta[0] * fprod[j];
+        beta[j+1] = beta[0] * fprod[j];
     }
 }
 //--------------------------------------------------------------------------
@@ -684,9 +590,9 @@ void getbetak (int n, int x, int nc, int jj,
 
 // return parameterisation cf Pledger et al. 2010 p 885 
 void getbetaB (int n, int x, int nc, int jj, 
-               const RMatrix<double> openval,  
-               const RVector<int> PIAJ, 
-               std::vector<double> &beta) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ, 
+    std::vector<double> &beta) {
     int j;
     double sumB = 0;
     std::vector<double> B(jj);
@@ -701,11 +607,11 @@ void getbetaB (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getbetaD (int n, int x, int nc, int jj, 
-               const RMatrix<double> openval,  
-               const RVector<int> PIAJ, 
-               std::vector<double> &phij,
-               std::vector<double> &beta) {
-        int j;
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ, 
+    std::vector<double> &phij,
+    std::vector<double> &beta) {
+    int j;
     double sumB;
     std::vector<double> B(jj);
     std::vector<double> D(jj);
@@ -724,26 +630,26 @@ void getbetaD (int n, int x, int nc, int jj,
 //--------------------------------------------------------------------------
 
 void getbeta (int type, int n, int x, int nc, int jj, 
-	      const RMatrix<double> openval,  
-	      const RVector<int> PIAJ, 
-	      const RVector<double> intervals,
-	      std::vector<double> &phij,
-	      std::vector<double> &beta) {
+    const RMatrix<double> openval,  
+    const RVector<int> PIAJ, 
+    const RVector<double> intervals,
+    std::vector<double> &phij,
+    std::vector<double> &beta) {
     if ((type == 2) || (type == 17) || (type == 11) || (type == 13) || 
-	(type == 41) || (type == 43) || (type == 30) || (type == 31)) 
+        (type == 41) || (type == 43) || (type == 30) || (type == 31)) 
         getbeta0 (n, x, nc, jj, openval, PIAJ, beta);
     else if ((type == 4) || (type == 15) ||  (type == 27) ||  (type == 7) || 
-             (type == 9) || (type == 37) || (type == 39))
+        (type == 9) || (type == 37) || (type == 39))
         getbetaf (n, x, nc, jj, openval, PIAJ, phij, intervals, beta);
     else if ((type == 3) || (type == 16) || (type == 10) || (type == 12) || 
-             (type == 20) || (type == 40) || (type == 42))
+        (type == 20) || (type == 40) || (type == 42))
         getbetal (n, x, nc, jj, openval, PIAJ, phij, intervals, beta);
     else if ((type == 14) || (type == 18))
         getbetaB (n, x, nc, jj, openval, PIAJ, beta);
     else if ((type == 8) || (type == 19) || (type == 38))
         getbetaD (n, x, nc, jj, openval, PIAJ, phij, beta);
     else if ((type == 22) || (type == 23) || (type == 24) || (type == 25) ||
-             (type == 26))
+        (type == 26))
         getbetag (n, x, nc, jj, openval, PIAJ, phij, intervals, beta);
     else if ((type == 28) || (type == 29))
         getbetak (n, x, nc, jj, openval, PIAJ, phij, beta);
