@@ -36,6 +36,7 @@ void convolvemq (
 void fillkernelp (int kn, 
                   int jj, 
                   int kerneltype, 
+                  bool sparsekernel,
                   double cellsize,
                   const RMatrix<int> kernel, 
                   const RVector<int> moveargsi, 
@@ -49,6 +50,7 @@ void fillkernelp (int kn,
 void fillkernelparallel (int kn, 
                          int jj, 
                          int kerneltype, 
+                         bool sparsekernel,
                          double cellsize,
                          const RMatrix<int> kernel, 
                          const RVector<int> moveargsi, 
@@ -64,6 +66,14 @@ double hfn
      int sigmai, int detectfn);
 //--------------------------------------------------------------------------
 
+double hfnd
+    (int k, int m, int c, 
+        const RMatrix<double> openval,  
+        const RMatrix<double> distmat,
+        int sigmai, 
+        int detectfn);
+//--------------------------------------------------------------------------
+
 void getp (int n, int x, int nc, int ss, 
            const RMatrix<double> openval,  
            const RVector<int> PIA, 
